@@ -43,6 +43,7 @@
 #include <geogram/basic/common.h>
 #include <geogram/numerics/multi_precision.h>
 #include <geogram/basic/matrix.h>
+#include <geogram/basic/rationalg.h>
 
 /**
  * \file geogram/numerics/expansion_nt.h
@@ -55,7 +56,6 @@
 namespace GEO {
 
     class expansion_nt;
-    class rational_nt;
     
    /**
      * \brief Expansion_nt (expansion Number Type) is used to compute the
@@ -69,14 +69,6 @@ namespace GEO {
      */
     class GEOGRAM_API expansion_nt {
     public:
-        /**
-         * \brief This type is used to overload expression_nt 
-         *  constructors with a version that does not create
-         *  an expansion.
-         */
-         enum UninitializedType {
-             UNINITIALIZED
-         };
 
         /**
          * \brief This type is used by the constructor that 
@@ -89,17 +81,14 @@ namespace GEO {
         /**
          * \brief Constructs an uninitialized expansion_nt.
          */
-         explicit expansion_nt(
-             UninitializedType uninitialized
-         ) : rep_(nullptr) {
-             geo_argused(uninitialized);
+         expansion_nt() : rep_(nullptr) {
          }
          
         /**
          * \brief Constructs a new expansion_nt from a double.
          * \param[in] x the value to initialize this expansion.
          */
-        explicit expansion_nt(double x = 0.0) {
+        explicit expansion_nt(double x) {
             rep_ = expansion::new_expansion_on_heap(1);
             rep()[0] = x;
             rep().set_length(1);
@@ -113,10 +102,7 @@ namespace GEO {
          */
         explicit expansion_nt(const expansion& rhs) {
             rep_ = expansion::new_expansion_on_heap(rhs.length());
-            rep().set_length(rhs.length());
-            for(index_t i=0; i<rhs.length(); ++i) {
-                rep()[i] = rhs[i];
-            }
+            rep().assign(rhs);
         }
 
         /**
@@ -222,6 +208,7 @@ namespace GEO {
                 geo_assert_not_reached;
                 break;
             case PRODUCT:
+                // HERE: TODO CHECK SIZE
                 const expansion& p1 = expansion_product(x,y);
                 const expansion& p2 = expansion_product(z,t);
                 rep_ = expansion::new_expansion_on_heap(
@@ -329,6 +316,13 @@ namespace GEO {
         void optimize() {
             rep().optimize();
         }
+
+        /**
+         * \brief Flips the sign of an expansion.
+         */
+        void negate() {
+            rep().negate();
+        }
         
         /********************************************************************/
 
@@ -669,7 +663,7 @@ namespace GEO {
             const double* a, const double* b, const double* c,
             coord_index_t dim
         );
-        friend class rational_nt;
+        // friend class rational_nt;
     };
 
     /**
@@ -831,6 +825,19 @@ namespace GEO {
         return x.sign();
     }
 
+    /**
+     * \brief Specialization of geo_cmp() for expansion_nt.
+     * \param x , y const references to two expansion_nt
+     * \retval POSITIVE if x > y
+     * \retval ZERO if x == y
+     * \retval NEGATIVE if x < y
+     */
+    template <> inline Sign geo_cmp(
+        const expansion_nt& x, const expansion_nt& y
+    ) {
+        return x.compare(y);
+    }
+    
     /************************************************************************/
 
     /**
@@ -928,6 +935,9 @@ namespace GEO {
         const expansion_nt& a32,const expansion_nt& a33 
     );
 
+// Make things a bit faster if target OS has large stack size
+#ifdef GEO_HAS_BIG_STACK
+    
     /**
      * \brief Specialization of det2x2 
      * \details Calls the optimized implementation for expansion_nt
@@ -986,6 +996,7 @@ namespace GEO {
         );
     }
     
+#endif
     
     /************************************************************************/
 }
@@ -1022,682 +1033,30 @@ inline std::istream& operator>> ( std::istream& is, GEO::expansion_nt& a) {
 
 namespace GEO {
 
-    /**
-     * \brief Rational_nt (rational Number Type) is used to compute the
-     *  sign of rational fractions exactly.
-     * \details Rational_nt can be used like float and double. It supports
-     *  four arithmetic operations (+,-,*,/), comparisons (>,>=,<,<=,==,!=)
-     *  and exact sign computation. When performance is a concern, the 
-     *  lower-level expansion class may be used instead.
-     */
-    class GEOGRAM_API rational_nt {
-      public:
-
-        /**
-         * \brief This type is used to overload expression_nt 
-         *  constructors with a version that does not create
-         *  an expansion.
-         */
-         enum UninitializedType {
-             UNINITIALIZED
-         };
-
-         /**
-          * \brief Constructs an uninitialized rational_nt.
-          */
-         explicit rational_nt(UninitializedType uninitialized) :
-            num_(expansion_nt::UNINITIALIZED),
-            denom_(expansion_nt::UNINITIALIZED) {
-             geo_argused(uninitialized);
-         }
-
-         
-        /**
-         * \brief Constructs a new rational_nt from a double.
-         * \param[in] x the value to initialize this rational_nt.
-         */
-        explicit rational_nt(double x = 0.0) : num_(x), denom_(1.0) {
-        }
-
-        /**
-         * \brief Constructs a new rational_nt from an expansion_nt.
-         * \param[in] x the value to initialize this rational_nt.
-         */
-        explicit rational_nt(const expansion_nt& x) : num_(x), denom_(1.0) {
-        }
-
-        /**
-         * \brief Constructs a new rational_nt from an expansion_nt
-         *  with move semantics
-         * \param[in] x the victim expansion_nt
-         */
-        explicit rational_nt(expansion_nt&& x) : num_(x), denom_(1.0) {
-        }
-        
-        /**
-         * \brief Constructs a new rational_nt from two doubles.
-         * \param[in] num the numerator
-	 * \param[in] denom the denominator
-         */
-        explicit rational_nt(double num, double denom)
-	    : num_(num), denom_(denom) {
-        }
-        
-        /**
-         * \brief Constructs a new rational_nt from two expansion_nt.
-         * \param[in] num the numerator
-	 * \param[in] denom the denominator
-         */
-        explicit rational_nt(const expansion_nt& num, const expansion_nt& denom)
-	    : num_(num), denom_(denom) {
-        }
-
-        /**
-         * \brief Constructs a new rational_nt from two expansion_nt with
-         *  move semantics
-         * \param[in] num the numerator
-	 * \param[in] denom the denominator
-         */
-        explicit rational_nt(
-            expansion_nt&& num, expansion_nt&& denom
-        ) : num_(num), denom_(denom) {
-        }
-            
-        /**
-         * \brief Copy-constructor.
-         * \param[in] rhs the rational to be copied
-         */
-        rational_nt(const rational_nt& rhs) {
-            copy(rhs);
-        }
-
-        /**
-         * \brief Move-constructor.
-         * \param[in] rhs the rational to be copied
-         */
-        rational_nt(rational_nt&& rhs) :
-           num_(rhs.num_),
-           denom_(rhs.denom_) {
-        }
-        
-        /**
-         * \brief Assignment operator.
-         * \param[in] rhs the rational to be copied
-         * \return the new value of this rational (rhs)
-         */
-        rational_nt& operator= (const rational_nt& rhs) {
-            num_ = rhs.num_;
-            denom_ = rhs.denom_;
-            return *this;
-        }
-
-        /**
-         * \brief Assignment operator with move semantics
-         * \param[in] rhs the victim rational_nt
-         * \return the new value of this rational (rhs)
-         */
-        rational_nt& operator= (rational_nt&& rhs) {
-            num_ = rhs.num_;
-            denom_ = rhs.denom_;
-            return *this;
-        }
-        
-	/**
-	 * \brief gets the numerator.
-	 * \return a const reference to the numerator.
-	 */
-	const expansion_nt& num() const {
-	    return num_;
-	}
-
-	/**
-	 * \brief gets the denominator.
-	 * \return a const reference to the denominator.
-	 */
-	const expansion_nt& denom() const {
-	    return denom_;
-	}
-
-	/**
-	 * \brief gets the numerator.
-	 * \return a reference to the numerator.
-	 */
-	 expansion_nt& num() {
-	    return num_;
-	}
-
-	/**
-	 * \brief gets the denominator.
-	 * \return a reference to the denominator.
-	 */
-	 expansion_nt& denom() {
-	    return denom_;
-	}
-
-        /**
-         * \brief Optimizes the internal representation without changing the
-         *  represented value
-         * \details this function can reduce the length of an expansion
-         */
-        void optimize() {
-            num().optimize();
-            denom().optimize();
-        }
-         
-        /********************************************************************/
-
-        /**
-         * \brief Adds a rational_nt to this rational_nt
-         * \param[in] rhs the rational_nt to be added to this rational_nt
-         * \return the new value of this rational_nt
-         */
-        rational_nt& operator+= (const rational_nt& rhs) {
-	    if(has_same_denom(rhs)) {
-		num_ += rhs.num_;
-	    } else {
-		num_ = num_ * rhs.denom_ + rhs.num_ * denom_;	    
-		denom_ *= rhs.denom_;
-	    }
-	    return *this;
-	}
-
-        /**
-         * \brief Subtracts a rational_nt to this rational_nt
-         * \param[in] rhs the rational_nt to be subtracted
-         * \return the new value of this rational_nt
-         */
-        rational_nt& operator-= (const rational_nt& rhs) {
-	    if(has_same_denom(rhs)) {
-		num_ -= rhs.num_;
-	    } else {
-		num_ = num_ * rhs.denom_ - rhs.num_ * denom_;	    
-		denom_ *= rhs.denom_;
-	    }
-	    return *this;
-	}
-
-        /**
-         * \brief Multiplies this rational_nt by a rational_nt
-         * \param[in] rhs the rational_nt to multiply this rational_nt by
-         * \return the new value of this rational_nt
-         */
-        rational_nt& operator*= (const rational_nt& rhs) {
-	    num_ *= rhs.num_;
-	    denom_ *= rhs.denom_;
-	    return *this;
-	}
-
-        /**
-         * \brief Divides this rational_nt by a rational_nt
-         * \param[in] rhs the rational_nt to divide this rational_nt by
-         * \return the new value of this rational_nt
-         */
-        rational_nt& operator/= (const rational_nt& rhs) {
-	    num_ *= rhs.denom_;
-	    denom_ *= rhs.num_;
-	    return *this;
-	}
-	
-        /**
-         * \brief Adds a double to this rational_nt
-         * \param[in] rhs the double to be added to this rational_nt
-         * \return the new value of this rational_nt
-         */
-        rational_nt& operator+= (double rhs) {
-	    num_ += denom_ * rhs;
-	    return *this;
-	}
-
-        /**
-         * \brief Subtracts a double from this rational_nt
-         * \param[in] rhs the double to be subtracted from this rational_nt
-         * \return the new value of this rational_nt
-         */
-        rational_nt& operator-= (double rhs) {
-	    num_ -= denom_ * rhs;
-	    return *this;
-	}
-
-        /**
-         * \brief Multiplies this rational_nt by a double
-         * \details If the double is a constant (possibly negative) power of
-         *  two (e.g. 0.125, 0.5, 2.0, 4.0 ...), one may use
-         *  num().scale_fast() / denom().scale_fast() instead.
-         * \param[in] rhs the double to multiply this rational_nt with
-         * \return the new value of this rational_nt
-         */
-        rational_nt& operator*= (double rhs) {
-	    num_ *= rhs;
-	    return *this;
-	}
-
-        /**
-         * \brief Divides this rational_nt by a double
-         * \details If the double is a constant (possibly negative) power of
-         *  two (e.g. 0.125, 0.5, 2.0, 4.0 ...), one may use
-         *  num().scale_fast() / denom().scale_fast() instead.
-         * \param[in] rhs the double to multiply this rational_nt with
-         * \return the new value of this rational_nt
-         */
-        rational_nt& operator/= (double rhs) {
-	    denom_ *= rhs;
-	    return *this;
-	}
-	
-        /********************************************************************/
-
-        /**
-         * \brief Computes the sum of two rational_nt%s
-         * \param[in] rhs the rational_nt to be added to this rational_nt
-         * \return the sum of this rational_nt and \p rhs
-         */
-        rational_nt operator+ (const rational_nt& rhs) const {
-	    if(has_same_denom(rhs)) {
-		return rational_nt(
-		    num_ + rhs.num_,
-		    denom_
-		);
-	    }
-	    return rational_nt(
-		num_ * rhs.denom_ + rhs.num_ * denom_,
-		denom_ * rhs.denom_
-	    );
-	}
-
-        /**
-         * \brief Computes the difference between two rational_nt%s
-         * \param[in] rhs the rational_nt to be subtracted from
-         *  this rational_nt
-         * \return the difference between this rational_nt and \p rhs
-         */
-        rational_nt operator- (const rational_nt& rhs) const {
-	    if(has_same_denom(rhs)) {
-		return rational_nt(
-		    num_ - rhs.num_,
-		    denom_
-		);
-	    }
-	    return rational_nt(
-		num_ * rhs.denom_ - rhs.num_ * denom_,
-		denom_ * rhs.denom_
-	    );
-	}
-
-        /**
-         * \brief Computes the product between two rational_nt%s
-         * \param[in] rhs the rational_nt to be multiplied by
-         *  this rational_nt
-         * \return the product between this rational_nt and \p rhs
-         */
-        rational_nt operator* (const rational_nt& rhs) const {
-	    return rational_nt(
-		num_ * rhs.num_,
-		denom_ * rhs.denom_
-	    );
-	}
-
-        /**
-         * \brief Computes the ratio between two rational_nt%s
-         * \param[in] rhs the rational_nt to be multiplied by
-         *  this rational_nt
-         * \return the ratio between this rational_nt and \p rhs
-         */
-        rational_nt operator/ (const rational_nt& rhs) const {
-	    return rational_nt(
-		num_ * rhs.denom_,
-		denom_ * rhs.num_
-	    );
-	}
-
-	
-        /**
-         * \brief Computes the sum of a rational_nt and a double.
-         * \param[in] rhs the double to be added to this rational_nt
-         * \return the sum of this rational_nt and \p rhs
-         */
-        rational_nt operator+ (double rhs) const {
-	    return rational_nt(
-		num_ + rhs * denom_,
-		denom_
-	    );
-	}
-
-        /**
-         * \brief Computes the difference between a rational_nt and a double.
-         * \param[in] rhs the double to be subtracted from this rational_nt
-         * \return the difference between this rational_nt and \p rhs
-         */
-        rational_nt operator- (double rhs) const {
-	    return rational_nt(
-		num_ - rhs * denom_,
-		denom_
-	    );
-	}
-
-        /**
-         * \brief Computes the product between a rational_nt and a double.
-         * \param[in] rhs the double to be multiplied by this rational_nt
-         * \return the product between this rational_nt and \p rhs
-         */
-        rational_nt operator* (double rhs) const {
-	    return rational_nt(
-		num_ *rhs,
-		denom_
-	    );
-	}
-
-        /**
-         * \brief Computes the ratio between a rational_nt and a double.
-         * \param[in] rhs the double to be multiplied by this rational_nt
-         * \return the ratio between this rational_nt and \p rhs
-         */
-        rational_nt operator/ (double rhs) const {
-	    return rational_nt(
-		num_,
-		denom_*rhs
-	    );
-	}
-	
-        /********************************************************************/
-
-        /**
-         * \brief Computes the opposite of this rational_nt.
-         * \return the opposite of this rational_nt
-         */
-        rational_nt operator- () const {
-	    return rational_nt(
-		-num_, 
-		denom_
-	    );
-	}
-
-        /********************************************************************/
-
-        /**
-         * \brief Compares two rational_nt
-         * \return the sign of this expansion minus rhs
-         */
-        Sign compare(const rational_nt& rhs) const;
-
-        /**
-         * \brief Compares a rational_nt with a double
-         * \return the sign of this expansion minus rhs
-         */
-        Sign compare(double rhs) const;
-        
-        /**
-         * \brief Compares this rational_nt with another one.
-         * \details Internally computes the sign of the difference
-         *  between this rational_nt and \p rhs.
-         * \return true if this rational_nt is greater than \p rhs,
-         *  false otherwise
-         */
-        bool operator> (const rational_nt& rhs) const {
-            return (int(compare(rhs))>0);
-        }
-
-        /**
-         * \brief Compares this rational_nt with another one.
-         * \details Internally computes the sign of the difference
-         *  between this rational_nt and \p rhs.
-         * \return true if this rational_nt is greater or equal than \p rhs,
-         *  false otherwise
-         */
-        bool operator>= (const rational_nt& rhs) const {
-            return (int(compare(rhs))>=0);            
-        }
-
-        /**
-         * \brief Compares this rational_nt with another one.
-         * \details Internally computes the sign of the difference
-         *  between this rational_nt and \p rhs.
-         * \return true if this rational_nt is smaller than \p rhs,
-         *  false otherwise
-         */
-        bool operator< (const rational_nt& rhs) const {
-            return (int(compare(rhs))<0);
-        }
-
-        /**
-         * \brief Compares this rational_nt with another one.
-         * \details Internally computes the sign of the difference
-         *  between this rational_nt and \p rhs.
-         * \return true if this rational_nt is smaller or equal than \p rhs,
-         *  false otherwise
-         */
-        bool operator<= (const rational_nt& rhs) const {
-            return (int(compare(rhs))<=0);
-        }
-
-        /**
-         * \brief Compares this rational_nt with another one.
-         * \details Internally computes the sign of the difference
-         *  between this rational_nt and \p rhs.
-         * \return true if this rational_nt is greater than \p rhs,
-         *  false otherwise
-         */
-        bool operator> (double rhs) const {
-            return (int(compare(rhs))>0);            
-        }
-
-        /**
-         * \brief Compares this rational_nt with another one.
-         * \details Internally computes the sign of the difference
-         *  between this rational_nt and \p rhs.
-         * \return true if this rational_nt is greater or equal than \p rhs,
-         *  false otherwise
-         */
-        bool operator>= (double rhs) const {
-            return (int(compare(rhs))>=0);            
-        }
-
-        /**
-         * \brief Compares this rational_nt with another one.
-         * \details Internally computes the sign of the difference
-         *  between this rational_nt and \p rhs.
-         * \return true if this rational_nt is smaller than \p rhs,
-         *  false otherwise
-         */
-        bool operator< (double rhs) const {
-            return (int(compare(rhs))<0);            
-        }
-
-        /**
-         * \brief Compares this rational_nt with another one.
-         * \details Internally computes the sign of the difference
-         *  between this rational_nt and \p rhs.
-         * \return true if this rational_nt is smaller or equal than \p rhs,
-         *  false otherwise
-         */
-        bool operator<= (double rhs) const {
-            return (int(compare(rhs))<=0);                        
-        }
-
-        /********************************************************************/
-
-        /**
-         * \brief Computes an approximation of the stored
-         *  value in this rational.
-         * \return an approximation of the stored value.
-         */
-        double estimate() const {
-            return num_.estimate() / denom_.estimate();
-        }
-        
-        /**
-         * \brief Gets the sign of this rational_nt.
-         * \return the sign of this rational_nt, computed exactly.
-         */
-        Sign sign() const {
-            geo_debug_assert(denom_.sign() != ZERO);
-            return Sign(num_.sign() * denom_.sign());
-        }
-
-      protected:
-        /**
-         * \brief Copies a rational into this one.
-         * \param[in] rhs a const reference to the rational to be copied
-         */
-	void copy(const rational_nt& rhs) {
-	    num_ = rhs.num_;
-	    denom_ = rhs.denom_;
-	}
-
-	/**
-	 * \brief Tests whether a rational_nt has trivially the same denominator
-	 *  as this rational_nt.
-	 * \details This function is used to implement faster addition, 
-	 *  subtraction and tests when it can be quickly determined that both
-	 *  operands have the same denominator.
-	 * \retval true if it is trivial that \p rhs has the same denominator
-	 *  as this rational_nt.
-	 * \retval false otherwise.
-	 */
-	bool has_same_denom(const rational_nt& rhs) const {
-            return denom_ == rhs.denom_;
-	}
-	
-      private:
-	expansion_nt num_;
-	expansion_nt denom_;
-    };
-
     /**************************************************************************/
 
-    /**
-     * \brief Computes the sum of a double and a rational_nt
-     * \param[in] a the double to be added
-     * \param[in] b the rational_nt to be added
-     * \return a rational_nt that represents \p a + \p b
-     * \relates rational_nt
-     */
-    inline rational_nt operator+ (double a, const rational_nt& b) {
-        return b + a;
-    }
+    namespace Numeric {
+        
+        template<> inline void optimize_number_representation(expansion_nt& x) {
+            x.optimize();
+        }
 
-    /**
-     * \brief Computes the difference between a double and a rational_nt
-     * \param[in] a the double
-     * \param[in] b the rational_nt to be subtracted
-     * \return a rational_nt that represents \p a - \p b
-     * \relates rational_nt
-     */
-    inline rational_nt operator- (double a, const rational_nt& b) {
-        rational_nt result = b - a;
-        result.num().rep().negate();
-        return result;
-    }
-
-    /**
-     * \brief Computes the product of a double and a rational_nt
-     * \param[in] a the double
-     * \param[in] b the rational_nt to be multiplied
-     * \return a rational_nt that represents \p a * \p b
-     * \relates rational_nt
-     */
-    inline rational_nt operator* (double a, const rational_nt& b) {
-        return b * a;
-    }
-
-    /**
-     * \brief Computes the ratio between a double and a rational_nt
-     * \param[in] a the double
-     * \param[in] b the rational_nt to be divided
-     * \return a rational_nt that represents \p a / \p b
-     * \relates rational_nt
-     */
-    inline rational_nt operator/ (double a, const rational_nt& b) {
-        return rational_nt(
-	    a*b.denom(),
-	    b.num()
-	);
+        /**
+         * \brief Compares two rational numbers given as separate
+         *   numerators and denominators. Specialization for exact_nt.
+         * \param[in] a_num , a_denom defines a = \p a_num / \p a_denom
+         * \param[in] b_num , b_denom defines b = \p b_num / \p b_denom
+         * \return the sign of a - b
+         */
+        template<> Sign GEOGRAM_API ratio_compare(
+            const expansion_nt& a_num, const expansion_nt& a_denom,
+            const expansion_nt& b_num, const expansion_nt& b_denom
+        );
     }
     
-    /**
-     * \brief Tests equality between two rational_nt%s.
-     * \details Implemented by testing whether the difference between
-     * \p a and \p b is 0.
-     * \return true if \p a and \p b represent exactly the same value, false
-     *  otherwise
-     * \relates rational_nt
-     */
-    inline bool operator== (const rational_nt& a, const rational_nt& b) {
-        return (a.compare(b) == ZERO);
-    }
-
-    /**
-     * \brief Tests equality between a rational_nt and a double.
-     * \details Implemented by testing whether the difference between
-     * \p a and \p b is 0.
-     * \return true if \p a and \p b represent exactly the same value, false
-     *  otherwise
-     * \relates rational_nt
-     */
-    inline bool operator== (const rational_nt& a, double b) {
-        return (a.compare(b) == ZERO);
-    }
-
-    /**
-     * \brief Tests equality between a double and a rational_nt.
-     * \details Implemented by testing whether the difference between
-     * \p a and \p b is 0.
-     * \return true if \p a and \p b represent exactly the same value, false
-     *  otherwise
-     * \relates rational_nt
-     */
-    inline bool operator== (double a, const rational_nt& b) {
-        return (b.compare(a) == ZERO);
-    }
-
-    /**
-     * \brief Tests whether two rational_nt%s differ.
-     * \details Implemented by testing whether the difference between
-     * \p a and \p b is different from 0.
-     * \return true if \p a and \p b do not represent the same exact value,
-     *  false otherwise
-     * \relates rational_nt
-     */
-    inline bool operator!= (const rational_nt& a, const rational_nt& b) {
-        return (a.compare(b) != ZERO);
-    }
-
-    /**
-     * \brief Tests whether a rational_nt differs from a double.
-     * \details Implemented by testing whether the difference between
-     * \p a and \p b is different from 0.
-     * \return true if \p a and \p b do not represent the same exact value,
-     *  false otherwise
-     * \relates rational_nt
-     */
-    inline bool operator!= (const rational_nt& a, double b) {
-        return (a.compare(b) != ZERO);
-    }
-
-    /**
-     * \brief Tests whether a double differs from a rational_nt.
-     * \details Implemented by testing whether the difference between
-     * \p a and \p b is different from 0.
-     * \return true if \p a and \p b do not represent the same exact value,
-     *  false otherwise
-     * \relates rational_nt
-     */
-    inline bool operator!= (double a, const rational_nt& b) {
-        return (b.compare(a) != ZERO);
-    }
-
     /**************************************************************************/
-    
-    /**
-     * \brief Specialization of geo_sgn() for rational_nt.
-     * \param x a const reference to a rational_nt
-     * \return the (exact) sign of x (one of POSITIVE, ZERO, NEGATIVE)
-     */
-    template <> inline Sign geo_sgn(const rational_nt& x) {
-        return x.sign();
-    }
 
-    /**************************************************************************/
-    
+    typedef rationalg<expansion_nt> rational_nt;
 }
 
 #endif
