@@ -287,6 +287,22 @@ namespace GEO {
                 (capa + 1) * sizeof(double);
         }
 
+        /**
+         * \brief Computes the amount of memory required to store
+         *  an expansion on the stack
+         * \details Behaves like bytes() but in debug mode checks
+         *  that this will fit on the stack. 
+         * \param[in] capa the required capacity
+         * \return the total number of bytes required to store
+         *  an expansion of capacity \p capa.
+         */
+        static size_t bytes_on_stack(index_t capa) {
+#ifndef GEO_HAS_BIG_STACK            
+            geo_debug_assert(capa < MAX_CAPACITY_ON_STACK);
+#endif            
+            return bytes(capa);
+        }
+        
         /**
          * \brief Client code should not use this constructor.
          * \details This constructor should not be used by client code,
@@ -315,7 +331,7 @@ namespace GEO {
     expansion& new_expansion_on_stack(index_t capa);         
 #else
 #define new_expansion_on_stack(capa)                           \
-    (new (alloca(expansion::bytes(capa)))expansion(capa))
+    (new (alloca(expansion::bytes_on_stack(capa)))expansion(capa))
 #endif
 
         /**
@@ -346,7 +362,34 @@ namespace GEO {
 	    x_[0] = a;
 	    return *this;
 	}
-	
+
+	/**
+	 * \brief Copies an expansion to this expansion
+	 * \param[in] rhs the expansion to be copied
+	 * \return the new value of this expansion (\p rhs)
+	 */
+	expansion& assign(const expansion& rhs) {
+            geo_debug_assert(capacity() >= rhs.length());
+	    set_length(rhs.length());
+            for(index_t i=0; i<rhs.length(); ++i) {
+                x_[i] = rhs.x_[i];
+            }
+	    return *this;
+	}
+
+	/**
+	 * \brief Copies the absolute value of an expansion to this expansion
+	 * \param[in] rhs the expansion to be copied
+	 * \return the new value of this expansion that is, abs(\p rhs)
+	 */
+        expansion& assign_abs(const expansion& rhs) {
+            assign(rhs);
+            if(sign() == NEGATIVE) {
+                negate();
+            }
+            return *this;
+        }
+        
         /**
          * \brief Computes the required capacity to store the
          *  sum of two doubles.
@@ -1089,6 +1132,11 @@ namespace GEO {
          * \details this function can reduce the length of an expansion
          */
         void optimize();
+
+        /**
+         * \brief Show global statistics
+         */
+        static void show_all_stats();
         
     protected:
         /**
@@ -1108,6 +1156,8 @@ namespace GEO {
 
         /**
          * \brief Assigns a sub-product to this expansion.
+         * \details Used by assign_product() when operating in balanced
+         *  distillation mode. Recursively assembles sub-sums.
          * \param[in] a a pointer to the first component of the first term
          * \param[in] a_length number of components in first term
          * \param[in] b second term
@@ -1117,39 +1167,18 @@ namespace GEO {
             const double* a, index_t a_length, const expansion& b
         );
 
-        /**
-         * \brief Computes an expansion that represents the exact
-         *  product of its arguments (a[0]+a[1]+ ... +a[length-1])*b.
-         *
-         * \details Used internally to implement product ("distillation", see
-         *  Shewchuk's paper).
-         *
-         * \param[in] a an expansion, specified as a const double*
-         * \param[in] a_length number of coefficients in a
-         * \param[in] b an expansion, specified as a const expansion&
-         *
-         * \return a reference to an expansion, allocated on the stack.
-         * \warning Do not return or use the returned reference outside the
-         * calling function.
-         * \relates GEO::expansion
-         */
-#define expansion_sub_product(a, a_length, b)           \
-    new_expansion_on_stack(                       \
-        sub_product_capacity(a_length, b.length()) \
-    )->assign_sub_product(a, a_length, b)
-
-    private:
         /**
          * \brief Expansion%s cannot be copied.
          */
-        expansion(const expansion& rhs);
+        expansion(const expansion& rhs) = delete;
 
         /**
          * \brief Expansion%s cannot be copied.
          */
-        expansion& operator= (const expansion& rhs);
+        expansion& operator= (const expansion& rhs) = delete;
 
     private:
+        static constexpr index_t MAX_CAPACITY_ON_STACK = 1023;
         index_t length_;
         index_t capacity_;
         double x_[2];  // x_ is in fact of size [capacity_]
@@ -1174,6 +1203,22 @@ namespace GEO {
 #define expansion_create(a)	      \
     new_expansion_on_stack(1)->assign(a)
 
+
+    /**
+     * \brief Creates an expansion from the absolute value of another
+     *  expansion
+     * \param[in] e the expansion
+     * \return a reference to an expansion, allocated on the
+     *  stack.
+     * \code
+     *  const expansion& e1 = expansion_abs(e);
+     * \endcode
+     * \warning Do not return or use the returned reference outside the
+     *  calling function.
+     * \relates GEO::expansion
+     */
+#define expansion_abs(e)	      \
+    new_expansion_on_stack(e.length())->assign_abs(e)
     
     /**
      * \brief Computes an expansion that represents the exact
